@@ -2,8 +2,9 @@
  * Copyright (c) 2004, 2005 Christophe Varoqui
  */
 #include <stdio.h>
-#include <checkers.h>
+#include <libudev.h>
 
+#include "checkers.h"
 #include "memory.h"
 #include "vector.h"
 #include "util.h"
@@ -16,7 +17,7 @@ extern int
 store_ble (vector blist, char * str, int origin)
 {
 	struct blentry * ble;
-	
+
 	if (!str)
 		return 0;
 
@@ -51,26 +52,26 @@ alloc_ble_device (vector blist)
 {
 	struct blentry_device * ble = MALLOC(sizeof(struct blentry_device));
 
-	if (!ble || !blist)
+	if (!ble)
 		return 1;
 
-	if (!vector_alloc_slot(blist)) {
+	if (!blist || !vector_alloc_slot(blist)) {
 		FREE(ble);
 		return 1;
 	}
 	vector_set_slot(blist, ble);
 	return 0;
 }
-	
+
 extern int
 set_ble_device (vector blist, char * vendor, char * product, int origin)
 {
 	struct blentry_device * ble;
-	
+
 	if (!blist)
 		return 1;
 
-	ble = VECTOR_SLOT(blist, VECTOR_SIZE(blist) - 1);
+	ble = VECTOR_LAST_SLOT(blist);
 
 	if (!ble)
 		return 1;
@@ -96,64 +97,20 @@ set_ble_device (vector blist, char * vendor, char * product, int origin)
 }
 
 int
-setup_default_blist (struct config * conf)
+_blacklist_exceptions (vector elist, const char * str)
 {
-	struct blentry * ble;
-	struct hwentry *hwe;
-	char * str;
 	int i;
+	struct blentry * ele;
 
-	str = STRDUP("^(ram|raw|loop|fd|md|dm-|sr|scd|st)[0-9]*");
-	if (!str)
-		return 1;
-	if (store_ble(conf->blist_devnode, str, ORIGIN_DEFAULT))
-		return 1;
-
-	str = STRDUP("^hd[a-z]");
-	if (!str)
-		return 1;
-	if (store_ble(conf->blist_devnode, str, ORIGIN_DEFAULT))
-		return 1;
-
-	str = STRDUP("^dcssblk[0-9]*");
-	if (!str)
-		return 1;
-	if (store_ble(conf->blist_devnode, str, ORIGIN_DEFAULT))
-		return 1;
-
-	vector_foreach_slot (conf->hwtable, hwe, i) {
-		if (hwe->bl_product) {
-			if (alloc_ble_device(conf->blist_device))
-				return 1;
-			ble = VECTOR_SLOT(conf->blist_device,
-					  VECTOR_SIZE(conf->blist_device) -1);
-			if (set_ble_device(conf->blist_device,
-					   STRDUP(hwe->vendor),
-					   STRDUP(hwe->bl_product),
-					   ORIGIN_DEFAULT)) {
-				FREE(ble);
-				return 1;
-			}
-		}
+	vector_foreach_slot (elist, ele, i) {
+		if (!regexec(&ele->regex, str, 0, NULL, 0))
+			return 1;
 	}
 	return 0;
 }
 
 int
-_blacklist_exceptions (vector elist, char * str)
-{
-        int i;
-        struct blentry * ele;
-
-        vector_foreach_slot (elist, ele, i) {
-                if (!regexec(&ele->regex, str, 0, NULL, 0))
-			return 1;
-	}
-        return 0;
-}
-
-int
-_blacklist (vector blist, char * str)
+_blacklist (vector blist, const char * str)
 {
 	int i;
 	struct blentry * ble;
@@ -193,16 +150,73 @@ _blacklist_device (vector blist, char * vendor, char * product)
 	return 0;
 }
 
-#define LOG_BLIST(M) \
-	if (vendor && product)                                           \
-		condlog(3, "%s: (%s:%s) %s", dev, vendor, product, (M)); \
-	else if (wwid)                                                   \
-		condlog(3, "%s: (%s) %s", dev, wwid, (M));               \
-	else                                                             \
-		condlog(3, "%s: %s", dev, (M))
+int
+setup_default_blist (struct config * conf)
+{
+	struct blentry * ble;
+	struct hwentry *hwe;
+	char * str;
+	int i;
+
+	str = STRDUP("^(ram|raw|loop|fd|md|dm-|sr|scd|st)[0-9]*");
+	if (!str)
+		return 1;
+	if (store_ble(conf->blist_devnode, str, ORIGIN_DEFAULT))
+		return 1;
+
+	str = STRDUP("^(td|hd)[a-z]");
+	if (!str)
+		return 1;
+	if (store_ble(conf->blist_devnode, str, ORIGIN_DEFAULT))
+		return 1;
+
+	str = STRDUP("^dcssblk[0-9]*");
+	if (!str)
+		return 1;
+	if (store_ble(conf->blist_devnode, str, ORIGIN_DEFAULT))
+		return 1;
+
+	str = STRDUP("(ID_SCSI_VPD|ID_WWN)");
+	if (!str)
+		return 1;
+	if (store_ble(conf->elist_property, str, ORIGIN_DEFAULT))
+		return 1;
+
+	vector_foreach_slot (conf->hwtable, hwe, i) {
+		if (hwe->bl_product) {
+			if (_blacklist_device(conf->blist_device, hwe->vendor,
+					      hwe->bl_product))
+				continue;
+			if (alloc_ble_device(conf->blist_device))
+				return 1;
+			ble = VECTOR_SLOT(conf->blist_device,
+					  VECTOR_SIZE(conf->blist_device) -1);
+			if (set_ble_device(conf->blist_device,
+					   STRDUP(hwe->vendor),
+					   STRDUP(hwe->bl_product),
+					   ORIGIN_DEFAULT)) {
+				FREE(ble);
+				return 1;
+			}
+		}
+	}
+	return 0;
+}
+
+#define LOG_BLIST(M,S)							\
+	if (vendor && product)						\
+		condlog(3, "%s: (%s:%s) %s %s",				\
+			dev, vendor, product, (M), (S));		\
+	else if (wwid)							\
+		condlog(3, "%s: %s %s %s", dev, (M), wwid, (S));	\
+	else if (env)							\
+		condlog(3, "%s: %s %s %s", dev, (M), env, (S));		\
+	else								\
+		condlog(3, "%s: %s %s", dev, (M), (S))
 
 void
-log_filter (char *dev, char *vendor, char *product, char *wwid, int r)
+log_filter (const char *dev, char *vendor, char *product, char *wwid,
+	    const char *env, int r)
 {
 	/*
 	 * Try to sort from most likely to least.
@@ -211,22 +225,31 @@ log_filter (char *dev, char *vendor, char *product, char *wwid, int r)
 	case MATCH_NOTHING:
 		break;
 	case MATCH_DEVICE_BLIST:
-		LOG_BLIST("vendor/product blacklisted");
+		LOG_BLIST("vendor/product", "blacklisted");
 		break;
 	case MATCH_WWID_BLIST:
-		LOG_BLIST("wwid blacklisted");
+		LOG_BLIST("wwid", "blacklisted");
 		break;
 	case MATCH_DEVNODE_BLIST:
-		LOG_BLIST("device node name blacklisted");
+		LOG_BLIST("device node name", "blacklisted");
+		break;
+	case MATCH_PROPERTY_BLIST:
+		LOG_BLIST("udev property", "blacklisted");
 		break;
 	case MATCH_DEVICE_BLIST_EXCEPT:
-		LOG_BLIST("vendor/product whitelisted");
+		LOG_BLIST("vendor/product", "whitelisted");
 		break;
 	case MATCH_WWID_BLIST_EXCEPT:
-		LOG_BLIST("wwid whitelisted");
+		LOG_BLIST("wwid", "whitelisted");
 		break;
 	case MATCH_DEVNODE_BLIST_EXCEPT:
-		LOG_BLIST("device node name whitelisted");
+		LOG_BLIST("device node name", "whitelisted");
+		break;
+	case MATCH_PROPERTY_BLIST_EXCEPT:
+		LOG_BLIST("udev property", "whitelisted");
+		break;
+	case MATCH_PROPERTY_BLIST_MISSING:
+		LOG_BLIST("blacklisted,", "udev property missing");
 		break;
 	}
 }
@@ -247,7 +270,7 @@ int
 filter_device (vector blist, vector elist, char * vendor, char * product)
 {
 	int r = _filter_device(blist, elist, vendor, product);
-	log_filter(NULL, vendor, product, NULL, r);
+	log_filter(NULL, vendor, product, NULL, NULL, r);
 	return r;
 }
 
@@ -267,7 +290,7 @@ int
 filter_devnode (vector blist, vector elist, char * dev)
 {
 	int r = _filter_devnode(blist, elist, dev);
-	log_filter(dev, NULL, NULL, NULL, r);
+	log_filter(dev, NULL, NULL, NULL, NULL, r);
 	return r;
 }
 
@@ -287,7 +310,7 @@ int
 filter_wwid (vector blist, vector elist, char * wwid)
 {
 	int r = _filter_wwid(blist, elist, wwid);
-	log_filter(NULL, NULL, NULL, wwid, r);
+	log_filter(NULL, NULL, NULL, wwid, NULL, r);
 	return r;
 }
 
@@ -297,26 +320,69 @@ _filter_path (struct config * conf, struct path * pp)
 	int r;
 
 	r = _filter_devnode(conf->blist_devnode, conf->elist_devnode,pp->dev);
-	if (r)
-		return r;
-	r = _filter_wwid(conf->blist_wwid, conf->elist_devnode, pp->wwid);
-	if (r)
+	if (r > 0)
 		return r;
 	r = _filter_device(conf->blist_device, conf->elist_device,
-		 	   pp->vendor_id, pp->product_id);
-	if (r)
+			   pp->vendor_id, pp->product_id);
+	if (r > 0)
 		return r;
-	return 0;
+	r = _filter_wwid(conf->blist_wwid, conf->elist_wwid, pp->wwid);
+	return r;
 }
 
 int
 filter_path (struct config * conf, struct path * pp)
 {
 	int r=_filter_path(conf, pp);
-	log_filter(pp->dev, pp->vendor_id, pp->product_id, pp->wwid, r);
+	log_filter(pp->dev, pp->vendor_id, pp->product_id, pp->wwid, NULL, r);
 	return r;
 }
 
+int
+_filter_property (struct config *conf, const char *env)
+{
+	if (_blacklist_exceptions(conf->elist_property, env))
+		return MATCH_PROPERTY_BLIST_EXCEPT;
+	if (_blacklist(conf->blist_property, env))
+		return MATCH_PROPERTY_BLIST;
+
+	return 0;
+}
+
+int
+filter_property(struct config * conf, struct udev_device * udev)
+{
+	const char *devname = udev_device_get_sysname(udev);
+	struct udev_list_entry *list_entry;
+	int r;
+
+	if (!udev)
+		return 0;
+
+	udev_list_entry_foreach(list_entry,
+				udev_device_get_properties_list_entry(udev)) {
+		const char *env;
+
+		env = udev_list_entry_get_name(list_entry);
+		if (!env)
+			continue;
+
+		r = _filter_property(conf, env);
+		if (r) {
+			log_filter(devname, NULL, NULL, NULL, env, r);
+			return r;
+		}
+	}
+
+	/*
+	 * This is the inverse of the 'normal' matching;
+	 * the environment variable _has_ to match.
+	 */
+	log_filter(devname, NULL, NULL, NULL, NULL,
+		   MATCH_PROPERTY_BLIST_MISSING);
+	return MATCH_PROPERTY_BLIST_MISSING;
+}
+
 void
 free_blacklist (vector blist)
 {
@@ -347,10 +413,14 @@ free_blacklist_device (vector blist)
 
 	vector_foreach_slot (blist, ble, i) {
 		if (ble) {
-			regfree(&ble->vendor_reg);
-			regfree(&ble->product_reg);
-			FREE(ble->vendor);
-			FREE(ble->product);
+			if (ble->vendor) {
+				regfree(&ble->vendor_reg);
+				FREE(ble->vendor);
+			}
+			if (ble->product) {
+				regfree(&ble->product_reg);
+				FREE(ble->product);
+			}
 			FREE(ble);
 		}
 	}
