@@ -1,6 +1,6 @@
 /*
  * Original author : tridge@samba.org, January 2002
- * 
+ *
  * Copyright (c) 2005 Christophe Varoqui
  * Copyright (c) 2005 Benjamin Marzinski, Redhat
  */
@@ -8,6 +8,7 @@
 /*
  * A simple domain socket listener
  */
+#define _GNU_SOURCE
 #include <stdio.h>
 #include <stdlib.h>
 #include <unistd.h>
@@ -19,19 +20,21 @@
 #include <sys/socket.h>
 #include <sys/un.h>
 #include <sys/poll.h>
-
+#include <signal.h>
 #include <checkers.h>
-
 #include <memory.h>
 #include <debug.h>
 #include <vector.h>
 #include <structs.h>
+#include <structs_vec.h>
 #include <uxsock.h>
 #include <defaults.h>
 
+#include "main.h"
+#include "cli.h"
 #include "uxlsnr.h"
 
-#define SLEEP_TIME 5000
+struct timespec sleep_time = {5, 0};
 
 struct client {
 	int fd;
@@ -40,6 +43,9 @@ struct client {
 
 static struct client *clients;
 static unsigned num_clients;
+struct pollfd *polls;
+volatile sig_atomic_t reconfig_sig = 0;
+volatile sig_atomic_t log_reset_sig = 0;
 
 /*
  * handle a new client joining
@@ -52,7 +58,7 @@ static void new_client(int ux_sock)
 	int fd;
 
 	fd = accept(ux_sock, &addr, &len);
-	
+
 	if (fd == -1)
 		return;
 
@@ -81,7 +87,14 @@ static void dead_client(struct client *c)
 
 void free_polls (void)
 {
-	FREE(polls);
+	if (polls)
+		FREE(polls);
+}
+
+void uxsock_cleanup(void *arg)
+{
+	cli_exit();
+	free_polls();
 }
 
 /*
@@ -95,16 +108,19 @@ void * uxsock_listen(int (*uxsock_trigger)(char *, char **, int *, void *),
 	int rlen;
 	char *inbuf;
 	char *reply;
+	sigset_t mask;
 
 	ux_sock = ux_socket_listen(DEFAULT_SOCKET);
 
-	if (ux_sock == -1) {
-		condlog(0, "ux_socket_listen error");
+	if (ux_sock == -1)
 		exit(1);
-	}
+
+	pthread_cleanup_push(uxsock_cleanup, NULL);
 
 	polls = (struct pollfd *)MALLOC(0);
-
+	pthread_sigmask(SIG_SETMASK, NULL, &mask);
+	sigdelset(&mask, SIGHUP);
+	sigdelset(&mask, SIGUSR1);
 	while (1) {
 		struct client *c;
 		int i, poll_count;
@@ -121,15 +137,17 @@ void * uxsock_listen(int (*uxsock_trigger)(char *, char **, int *, void *),
 		}
 
 		/* most of our life is spent in this call */
-		poll_count = poll(polls, i, SLEEP_TIME);
-		
+		poll_count = ppoll(polls, i, &sleep_time, &mask);
+
 		if (poll_count == -1) {
-			if (errno == EINTR)
+			if (errno == EINTR) {
+				handle_signals();
 				continue;
+			}
 
 			/* something went badly wrong! */
 			condlog(0, "poll");
-			exit(1);
+			pthread_exit(NULL);
 		}
 
 		if (poll_count == 0)
@@ -146,13 +164,14 @@ void * uxsock_listen(int (*uxsock_trigger)(char *, char **, int *, void *),
 					inbuf[len - 1] = 0;
 					condlog(4, "Got request [%s]", inbuf);
 					uxsock_trigger(inbuf, &reply, &rlen,
-							trigger_data);
-
+						       trigger_data);
 					if (reply) {
 						if (send_packet(c->fd, reply,
-						     rlen) != 0) {
+								rlen) != 0) {
 							dead_client(c);
 						}
+						condlog(4, "Reply [%d bytes]",
+							rlen);
 						FREE(reply);
 						reply = NULL;
 					}
@@ -168,6 +187,7 @@ void * uxsock_listen(int (*uxsock_trigger)(char *, char **, int *, void *),
 		}
 	}
 
+	pthread_cleanup_pop(1);
 	close(ux_sock);
 	return NULL;
 }
